html, body {
    width: 100%;
    height: 100%;
    padding: 0;
    margin: 0;
    background-color: black;
    overflow: hidden;
}

.pwnstar-tabs {
    height: 30px;
    background-color: #282828;
    user-select: none;
}

.pwnstar-tab {
    height: 30px;
    display: inline-block;
    background-color: gray;
    padding-left: 50px;
    padding-right: 50px;
    margin-right: 1px;
    border-radius: 7px 7px 0px 0px;
    font-family: monospace;
    text-align: center;
    vertical-align: middle;
    line-height: 30px;
    user-select: none;
}

.tab:focus {
    outline: none;
}

.pwnstar-terminal {
    width: calc(100% - 20px);
    height: calc(100% - 20px - 20px);
    visibility: hidden;
    margin: 10;
}

.modal {
    position: absolute;
    left: 50%;
    top: 50%;
    z-index: 1;
}

.loader {
    width: 120px;
    height: 120px;
    margin-left: -76px;
    margin-top: -76px;
    border: 16px solid #f3f3f3;
    border-top: 16px solid #3498db;
    border-radius: 50%;
    animation: spin 2s linear infinite;
}

@keyframes spin {
  0% { transform: rotate(0deg); }
  100% { transform: rotate(360deg); }
}

.redo {
    width: 152px;
    height: 152px;
    margin-left: -76px;
    margin-top: -76px;
    background-color: #f3f3f3;
    cursor: pointer;
    mask: url(/img/redo-solid.svg) no-repeat center;
    -webkit-mask: url(/img/redo-solid.svg) no-repeat center;
}

.xterm-viewport {
    overflow: hidden !important;
}
